import { IsOptional } from "class-validator";
import { ChildEntity, getConnectionManager, ManyToOne, OneToMany } from "typeorm";
import { Address } from "./Address";
import { Runner } from './Runner';
import { RunnerGroup } from "./RunnerGroup";
import { RunnerTeam } from "./RunnerTeam";

/**
 * Defines a runner organisation (business or school for example).
*/
@ChildEntity()
export class RunnerOrganisation extends RunnerGroup {

  /**
   * The organisations's address.
   * Optional
   */
  @IsOptional()
  @ManyToOne(() => Address, address => address.groups, { nullable: true })
  address?: Address;

  /**
 * Used to link teams to runner groups.
 */
  @OneToMany(() => RunnerTeam, team => team.parentGroup, { nullable: true })
  teams: RunnerTeam[];


  /**
   * Returns all runners associated with this organisation or it's teams.
   */
  public async getRunners() {
    let runners: Runner[] = new Array<Runner>();
    const teams = await this.getTeams();

    await teams.forEach(async team => {
      runners.push(... await team.getRunners());
    });
    await runners.push(... await getConnectionManager().get().getRepository(Runner).find({ group: this }));

    return runners;
  }

  /**
   * Returns all teams associated with this organisation.
   */
  public async getTeams() {
    return await getConnectionManager().get().getRepository(RunnerTeam).find({ parentGroup: this });
  }
}